@@ -26,6 +26,15 @@
  */
 
 #include "../include/Clip.h"
+#include "../include/FFmpegReader.h"
+#include "../include/FrameMapper.h"
+#ifdef USE_IMAGEMAGICK
+	#include "../include/ImageReader.h"
+	#include "../include/TextReader.h"
+#endif
+#include "../include/QtImageReader.h"
+#include "../include/ChunkReader.h"
+#include "../include/DummyReader.h"
 
 using namespace openshot;
 
@@ -41,6 +50,7 @@ void Clip::init_settings()
 	scale = SCALE_FIT;
 	anchor = ANCHOR_CANVAS;
 	display = FRAME_DISPLAY_NONE;
+	mixing = VOLUME_MIX_NONE;
 	waveform = false;
 	previous_properties = "";
 
@@ -52,9 +62,11 @@ void Clip::init_settings()
 	location_x = Keyframe(0.0);
 	location_y = Keyframe(0.0);
 
-	// Init alpha & rotation
+	// Init alpha
 	alpha = Keyframe(1.0);
-	rotation = Keyframe(0.0);
+
+	// Init rotation
+	init_reader_rotation();
 
 	// Init time & volume
 	time = Keyframe(1.0);
@@ -91,28 +103,46 @@ void Clip::init_settings()
 	has_video = Keyframe(-1.0);
 
 	// Default pointers
-	reader = NULL;
-	resampler = NULL;
-	audio_cache = NULL;
 	manage_reader = false;
 }
 
+// Init reader's rotation (if any)
+void Clip::init_reader_rotation() {
+	// Only init rotation from reader when needed
+	if (rotation.Points.size() > 1)
+		// Do nothing if more than 1 rotation Point
+		return;
+	else if (rotation.Points.size() == 1 && rotation.GetValue(1) != 0.0)
+		// Do nothing if 1 Point, and it's not the default value
+		return;
+
+	// Init rotation
+	if (reader && reader->info.metadata.count("rotate") > 0) {
+		// Use reader metadata rotation (if any)
+		// This is typical with cell phone videos filmed in different orientations
+		try {
+			float rotate_metadata = strtof(reader->info.metadata["rotate"].c_str(), 0);
+			rotation = Keyframe(rotate_metadata);
+		} catch (exception e) {}
+	}
+	else
+		// Default no rotation
+		rotation = Keyframe(0.0);
+}
+
 // Default Constructor for a clip
-Clip::Clip()
+Clip::Clip() : reader(NULL), resampler(NULL), audio_cache(NULL)
 {
 	// Init all default settings
 	init_settings();
 }
 
 // Constructor with reader
-Clip::Clip(ReaderBase* new_reader)
+Clip::Clip(ReaderBase* new_reader) : reader(new_reader), resampler(NULL), audio_cache(NULL)
 {
 	// Init all default settings
 	init_settings();
 
-	// Set the reader
-	reader = new_reader;
-
 	// Open and Close the reader (to set the duration of the clip)
 	Open();
 	Close();
@@ -122,7 +152,7 @@ Clip::Clip(ReaderBase* new_reader)
 }
 
 // Constructor with filepath
-Clip::Clip(string path)
+Clip::Clip(string path) : reader(NULL), resampler(NULL), audio_cache(NULL)
 {
 	// Init all default settings
 	init_settings();
@@ -165,6 +195,7 @@ Clip::Clip(string path)
 	if (reader) {
 		End(reader->info.duration);
 		manage_reader = true;
+		init_reader_rotation();
 	}
 }
 
@@ -189,6 +220,12 @@ void Clip::Reader(ReaderBase* new_reader)
 {
 	// set reader pointer
 	reader = new_reader;
+
+	// set parent
+	reader->SetClip(this);
+
+	// Init rotation (if any)
+	init_reader_rotation();
 }
 
 /// Get the current reader
@@ -303,13 +340,13 @@ std::shared_ptr<Frame> Clip::GetFrame(int64_t requested_frame)
 				frame->AddAudio(true, channel, 0, original_frame->GetAudioSamples(channel), original_frame->GetAudioSamplesCount(), 1.0);
 
 		// Get time mapped frame number (used to increase speed, change direction, etc...)
-		std::shared_ptr<Frame> new_frame = get_time_mapped_frame(frame, requested_frame);
+		get_time_mapped_frame(frame, requested_frame);
 
 		// Apply effects to the frame (if any)
-		apply_effects(new_frame);
+		apply_effects(frame);
 
 		// Return processed 'frame'
-		return new_frame;
+		return frame;
 	}
 	else
 		// Throw error if reader not initialized
@@ -352,7 +389,7 @@ void Clip::reverse_buffer(juce::AudioSampleBuffer* buffer)
 }
 
 // Adjust the audio and image of a time mapped frame
-std::shared_ptr<Frame> Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_number)
+void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_number)
 {
 	// Check for valid reader
 	if (!reader)
@@ -363,7 +400,6 @@ std::shared_ptr<Frame> Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame,
 	if (time.Values.size() > 1)
 	{
 		const GenericScopedLock<CriticalSection> lock(getFrameCriticalSection);
-		std::shared_ptr<Frame> new_frame;
 
 		// create buffer and resampler
 		juce::AudioSampleBuffer *samples = NULL;
@@ -371,15 +407,7 @@ std::shared_ptr<Frame> Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame,
 			resampler = new AudioResampler();
 
 		// Get new frame number
-		int new_frame_number = adjust_frame_number_minimum(round(time.GetValue(frame_number)));
-
-		// Create a new frame
-		int samples_in_frame = Frame::GetSamplesPerFrame(new_frame_number, reader->info.fps, reader->info.sample_rate, frame->GetAudioChannelsCount());
-		new_frame = std::make_shared<Frame>(new_frame_number, 1, 1, "#000000", samples_in_frame, frame->GetAudioChannelsCount());
-
-		// Copy the image from the new frame
-		new_frame->AddImage(GetOrCreateFrame(new_frame_number)->GetImage());
-
+		int new_frame_number = frame->number;
 
 		// Get delta (difference in previous Y value)
 		int delta = int(round(time.GetDelta(frame_number)));
@@ -427,7 +455,7 @@ std::shared_ptr<Frame> Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame,
 					start -= 1;
 				for (int channel = 0; channel < channels; channel++)
 					// Add new (slower) samples, to the frame object
-					new_frame->AddAudio(true, channel, 0, resampled_buffer->getReadPointer(channel, start),
+					frame->AddAudio(true, channel, 0, resampled_buffer->getReadPointer(channel, start),
 										number_of_samples, 1.0f);
 
 				// Clean up
@@ -535,7 +563,7 @@ std::shared_ptr<Frame> Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame,
 				// Add the newly resized audio samples to the current frame
 				for (int channel = 0; channel < channels; channel++)
 					// Add new (slower) samples, to the frame object
-					new_frame->AddAudio(true, channel, 0, buffer->getReadPointer(channel), number_of_samples, 1.0f);
+					frame->AddAudio(true, channel, 0, buffer->getReadPointer(channel), number_of_samples, 1.0f);
 
 				// Clean up
 				buffer = NULL;
@@ -556,7 +584,7 @@ std::shared_ptr<Frame> Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame,
 
 				// Add reversed samples to the frame object
 				for (int channel = 0; channel < channels; channel++)
-					new_frame->AddAudio(true, channel, 0, samples->getReadPointer(channel), number_of_samples, 1.0f);
+					frame->AddAudio(true, channel, 0, samples->getReadPointer(channel), number_of_samples, 1.0f);
 
 
 			}
@@ -564,13 +592,7 @@ std::shared_ptr<Frame> Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame,
 			delete samples;
 			samples = NULL;
 		}
-
-		// Return new time mapped frame
-		return new_frame;
-
-	} else
-		// Use original frame
-		return frame;
+	}
 }
 
 // Adjust frame number minimum value
@@ -596,35 +618,6 @@ std::shared_ptr<Frame> Clip::GetOrCreateFrame(int64_t number)
 		// Debug output
 		ZmqLogger::Instance()->AppendDebugMethod("Clip::GetOrCreateFrame (from reader)", "number", number, "samples_in_frame", samples_in_frame, "", -1, "", -1, "", -1, "", -1);
 
-		// Determine the max size of this clips source image (based on the timeline's size, the scaling mode,
-		// and the scaling keyframes). This is a performance improvement, to keep the images as small as possible,
-		// without loosing quality. NOTE: We cannot go smaller than the timeline itself, or the add_layer timeline
-        // method will scale it back to timeline size before scaling it smaller again. This needs to be fixed in
-        // the future.
-		if (scale == SCALE_FIT || scale == SCALE_STRETCH) {
-			// Best fit or Stretch scaling (based on max timeline size * scaling keyframes)
-			float max_scale_x = scale_x.GetMaxPoint().co.Y;
-			float max_scale_y = scale_y.GetMaxPoint().co.Y;
-			reader->SetMaxSize(max(float(max_width), max_width * max_scale_x), max(float(max_height), max_height * max_scale_y));
-
-		} else if (scale == SCALE_CROP) {
-			// Cropping scale mode (based on max timeline size * cropped size * scaling keyframes)
-			float max_scale_x = scale_x.GetMaxPoint().co.Y;
-			float max_scale_y = scale_y.GetMaxPoint().co.Y;
-			QSize width_size(max_width * max_scale_x, round(max_width / (float(reader->info.width) / float(reader->info.height))));
-			QSize height_size(round(max_height / (float(reader->info.height) / float(reader->info.width))), max_height * max_scale_y);
-
-			// respect aspect ratio
-			if (width_size.width() >= max_width && width_size.height() >= max_height)
-				reader->SetMaxSize(max(max_width, width_size.width()), max(max_height, width_size.height()));
-			else
-				reader->SetMaxSize(max(max_width, height_size.width()), max(max_height, height_size.height()));
-
-		} else {
-			// No scaling, use original image size (slower)
-			reader->SetMaxSize(0, 0);
-		}
-
 		// Attempt to get a frame (but this could fail if a reader has just been closed)
 		new_frame = reader->GetFrame(number);
 
@@ -647,6 +640,7 @@ std::shared_ptr<Frame> Clip::GetOrCreateFrame(int64_t number)
 	new_frame = std::make_shared<Frame>(number, reader->info.width, reader->info.height, "#000000", samples_in_frame, reader->info.channels);
 	new_frame->SampleRate(reader->info.sample_rate);
 	new_frame->ChannelsLayout(reader->info.channel_layout);
+	new_frame->AddAudioSilence(samples_in_frame);
 	return new_frame;
 }
 
@@ -670,8 +664,8 @@ string Clip::PropertiesJSON(int64_t requested_frame) {
 	root["duration"] = add_property_json("Duration", Duration(), "float", "", NULL, 0, 30 * 60 * 60 * 48, true, requested_frame);
 	root["gravity"] = add_property_json("Gravity", gravity, "int", "", NULL, 0, 8, false, requested_frame);
 	root["scale"] = add_property_json("Scale", scale, "int", "", NULL, 0, 3, false, requested_frame);
-	root["anchor"] = add_property_json("Anchor", anchor, "int", "", NULL, 0, 1, false, requested_frame);
 	root["display"] = add_property_json("Frame Number", display, "int", "", NULL, 0, 3, false, requested_frame);
+	root["mixing"] = add_property_json("Volume Mixing", mixing, "int", "", NULL, 0, 2, false, requested_frame);
 	root["waveform"] = add_property_json("Waveform", waveform, "int", "", NULL, 0, 1, false, requested_frame);
 
 	// Add gravity choices (dropdown style)
@@ -691,16 +685,17 @@ string Clip::PropertiesJSON(int64_t requested_frame) {
 	root["scale"]["choices"].append(add_property_choice_json("Stretch", SCALE_STRETCH, scale));
 	root["scale"]["choices"].append(add_property_choice_json("None", SCALE_NONE, scale));
 
-	// Add anchor choices (dropdown style)
-	root["anchor"]["choices"].append(add_property_choice_json("Canvas", ANCHOR_CANVAS, anchor));
-	root["anchor"]["choices"].append(add_property_choice_json("Viewport", ANCHOR_VIEWPORT, anchor));
-
 	// Add frame number display choices (dropdown style)
 	root["display"]["choices"].append(add_property_choice_json("None", FRAME_DISPLAY_NONE, display));
 	root["display"]["choices"].append(add_property_choice_json("Clip", FRAME_DISPLAY_CLIP, display));
 	root["display"]["choices"].append(add_property_choice_json("Timeline", FRAME_DISPLAY_TIMELINE, display));
 	root["display"]["choices"].append(add_property_choice_json("Both", FRAME_DISPLAY_BOTH, display));
 
+	// Add volume mixing choices (dropdown style)
+	root["mixing"]["choices"].append(add_property_choice_json("None", VOLUME_MIX_NONE, mixing));
+	root["mixing"]["choices"].append(add_property_choice_json("Average", VOLUME_MIX_AVERAGE, mixing));
+	root["mixing"]["choices"].append(add_property_choice_json("Reduce", VOLUME_MIX_REDUCE, mixing));
+
 	// Add waveform choices (dropdown style)
 	root["waveform"]["choices"].append(add_property_choice_json("Yes", true, waveform));
 	root["waveform"]["choices"].append(add_property_choice_json("No", false, waveform));
@@ -740,6 +735,7 @@ Json::Value Clip::JsonValue() {
 	root["scale"] = scale;
 	root["anchor"] = anchor;
 	root["display"] = display;
+	root["mixing"] = mixing;
 	root["waveform"] = waveform;
 	root["scale_x"] = scale_x.JsonValue();
 	root["scale_y"] = scale_y.JsonValue();
@@ -826,6 +822,8 @@ void Clip::SetJsonValue(Json::Value root) {
 		anchor = (AnchorType) root["anchor"].asInt();
 	if (!root["display"].isNull())
 		display = (FrameDisplayType) root["display"].asInt();
+	if (!root["mixing"].isNull())
+		mixing = (VolumeMixType) root["mixing"].asInt();
 	if (!root["waveform"].isNull())
 		waveform = root["waveform"].asBool();
 	if (!root["scale_x"].isNull())
@@ -897,13 +895,14 @@ void Clip::SetJsonValue(Json::Value root) {
 
 			if (!existing_effect["type"].isNull()) {
 				// Create instance of effect
-				e = EffectInfo().CreateEffect(existing_effect["type"].asString());
+				if (e = EffectInfo().CreateEffect(existing_effect["type"].asString())) {
 
-				// Load Json into Effect
-				e->SetJsonValue(existing_effect);
+					// Load Json into Effect
+					e->SetJsonValue(existing_effect);
 
-				// Add Effect to Timeline
-				AddEffect(e);
+					// Add Effect to Timeline
+					AddEffect(e);
+				}
 			}
 		}
 	}
@@ -966,9 +965,11 @@ void Clip::SetJsonValue(Json::Value root) {
 				reader->SetJsonValue(root["reader"]);
 			}
 
-			// mark as managed reader
-			if (reader)
+			// mark as managed reader and set parent
+			if (reader) {
+				reader->SetClip(this);
 				manage_reader = true;
+			}
 
 			// Re-Open reader (if needed)
 			if (already_open)
