@@ -55,16 +55,24 @@ FFmpegWriter::FFmpegWriter(string path) :
 // Open the writer
 void FFmpegWriter::Open()
 {
-	// Open the writer
-	is_open = true;
+	if (!is_open) {
+		// Open the writer
+		is_open = true;
 
-	// Prepare streams (if needed)
-	if (!prepare_streams)
-		PrepareStreams();
+		// Prepare streams (if needed)
+		if (!prepare_streams)
+			PrepareStreams();
 
-	// Write header (if needed)
-	if (!write_header)
-		WriteHeader();
+		// Now that all the parameters are set, we can open the audio and video codecs and allocate the necessary encode buffers
+		if (info.has_video && video_st)
+			open_video(oc, video_st);
+		if (info.has_audio && audio_st)
+			open_audio(oc, audio_st);
+
+		// Write header (if needed)
+		if (!write_header)
+			WriteHeader();
+	}
 }
 
 // auto detect format (from path)
@@ -146,7 +154,9 @@ void FFmpegWriter::SetVideoOptions(bool has_video, string codec, Fraction fps, i
 		info.pixel_ratio.num = pixel_ratio.num;
 		info.pixel_ratio.den = pixel_ratio.den;
 	}
-	if (bit_rate >= 1000)
+	if (bit_rate >= 1000)			// bit_rate is the bitrate in b/s
+		info.video_bit_rate = bit_rate;
+	if ((bit_rate >= 0) && 	(bit_rate < 64)	)	// bit_rate is the bitrate in crf
 		info.video_bit_rate = bit_rate;
 
 	info.interlaced_frame = interlaced;
@@ -237,7 +247,8 @@ void FFmpegWriter::SetOption(StreamType stream, string name, string value)
 
 	// Was option found?
 	if (option || (name == "g" || name == "qmin" || name == "qmax" || name == "max_b_frames" || name == "mb_decision" ||
-			       name == "level" || name == "profile" || name == "slices" || name == "rc_min_rate"  || name == "rc_max_rate"))
+			       name == "level" || name == "profile" || name == "slices" || name == "rc_min_rate"  || name == "rc_max_rate" ||
+					 	 name == "crf"))
 	{
 		// Check for specific named options
 		if (name == "g")
@@ -284,6 +295,60 @@ void FFmpegWriter::SetOption(StreamType stream, string name, string value)
 			// Buffer size
 			convert >> c->rc_buffer_size;
 
+		else if (name == "crf") {
+			// encode quality and special settings like lossless
+			// This might be better in an extra methods as more options
+			// and way to set quality are possible
+			#if  LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55, 39, 101)
+					switch (c->codec_id) {
+						#if (LIBAVCODEC_VERSION_MAJOR >= 58)
+						case AV_CODEC_ID_AV1 :
+							c->bit_rate = 0;
+							av_opt_set_int(c->priv_data, "crf", min(stoi(value),63), 0);
+							break;
+						#endif
+						case AV_CODEC_ID_VP8 :
+							c->bit_rate = 10000000;
+							av_opt_set_int(c->priv_data, "crf", max(min(stoi(value),63),4), 0); // 4-63
+							break;
+						case AV_CODEC_ID_VP9 :
+							c->bit_rate = 0;		// Must be zero!
+							av_opt_set_int(c->priv_data, "crf", min(stoi(value),63), 0); // 0-63
+							if (stoi(value) == 0) {
+								av_opt_set(c->priv_data, "preset", "veryslow", 0);
+								av_opt_set_int(c->priv_data, "lossless", 1, 0);
+							 }
+							 break;
+						case AV_CODEC_ID_H264 :
+							av_opt_set_int(c->priv_data, "crf", min(stoi(value),51), 0); // 0-51
+							if (stoi(value) == 0) {
+								av_opt_set(c->priv_data, "preset", "veryslow", 0);
+						 	}
+							break;
+						case AV_CODEC_ID_H265 :
+							av_opt_set_int(c->priv_data, "crf", min(stoi(value),51), 0); // 0-51
+							if (stoi(value) == 0) {
+								av_opt_set(c->priv_data, "preset", "veryslow", 0);
+								av_opt_set_int(c->priv_data, "lossless", 1, 0);
+						 	}
+							break;
+						default:
+							// If this codec doesn't support crf calculate a bitrate
+							// TODO: find better formula
+							double mbs = 15000000.0;
+							if (info.video_bit_rate > 0) {
+								if (info.video_bit_rate > 42) {
+									mbs = 380.0;
+								}
+								else {
+									mbs *= pow(0.912,info.video_bit_rate);
+								}
+							}
+							c->bit_rate = (int)(mbs);
+					}
+			#endif
+		}
+
 		else
 			// Set AVOption
 			AV_OPTION_SET(st, c->priv_data, name.c_str(), value.c_str(), c);
@@ -319,12 +384,6 @@ void FFmpegWriter::PrepareStreams()
 	// Initialize the streams (i.e. add the streams)
 	initialize_streams();
 
-	// Now that all the parameters are set, we can open the audio and video codecs and allocate the necessary encode buffers
-	if (info.has_video && video_st)
-		open_video(oc, video_st);
-	if (info.has_audio && audio_st)
-		open_audio(oc, audio_st);
-
 	// Mark as 'prepared'
 	prepare_streams = true;
 }
@@ -934,7 +993,19 @@ AVStream* FFmpegWriter::add_video_stream()
 #endif
 
 	/* Init video encoder options */
-	c->bit_rate = info.video_bit_rate;
+	if (info.video_bit_rate >= 1000) {
+		c->bit_rate = info.video_bit_rate;
+		if (info.video_bit_rate >= 1500000) {
+			c->qmin = 2;
+			c->qmax = 30;
+		}
+		// Here should be the setting for low fixed bitrate
+		// Defaults are used because mpeg2 otherwise had problems
+	}
+	else {
+		c->qmin = 0;
+		c->qmax = 63;
+	}
 
 	//TODO: Implement variable bitrate feature (which actually works). This implementation throws
 	//invalid bitrate errors and rc buffer underflow errors, etc...
@@ -943,8 +1014,6 @@ AVStream* FFmpegWriter::add_video_stream()
 	//c->rc_buffer_size = FFMAX(c->rc_max_rate, 15000000) * 112L / 15000000 * 16384;
 	//if ( !c->rc_initial_buffer_occupancy )
 	//	c->rc_initial_buffer_occupancy = c->rc_buffer_size * 3/4;
-	c->qmin = 2;
-	c->qmax = 30;
 
 	/* resolution must be a multiple of two */
 	// TODO: require /2 height and width
@@ -1383,7 +1452,7 @@ void FFmpegWriter::write_audio_packets(bool final)
 
 			} else {
 				// Create a new array
-				final_samples = new int16_t[audio_input_position * (av_get_bytes_per_sample(audio_codec->sample_fmt) / av_get_bytes_per_sample(AV_SAMPLE_FMT_S16))];
+				final_samples = (int16_t*)av_malloc(sizeof(int16_t) * audio_input_position * (av_get_bytes_per_sample(audio_codec->sample_fmt) / av_get_bytes_per_sample(AV_SAMPLE_FMT_S16)));
 
 				// Copy audio into buffer for frame
 				memcpy(final_samples, samples, audio_input_position * av_get_bytes_per_sample(audio_codec->sample_fmt));
@@ -1748,11 +1817,16 @@ void FFmpegWriter::OutputStreamInfo()
 // Init a collection of software rescalers (thread safe)
 void FFmpegWriter::InitScalers(int source_width, int source_height)
 {
+	int scale_mode = SWS_FAST_BILINEAR;
+	if (openshot::Settings::Instance()->HIGH_QUALITY_SCALING) {
+		scale_mode = SWS_LANCZOS;
+	}
+
 	// Init software rescalers vector (many of them, one for each thread)
 	for (int x = 0; x < num_of_rescalers; x++)
 	{
 		// Init the software scaler from FFMpeg
-		img_convert_ctx = sws_getContext(source_width, source_height, PIX_FMT_RGBA, info.width, info.height, AV_GET_CODEC_PIXEL_FORMAT(video_st, video_st->codec), SWS_LANCZOS, NULL, NULL, NULL);
+		img_convert_ctx = sws_getContext(source_width, source_height, PIX_FMT_RGBA, info.width, info.height, AV_GET_CODEC_PIXEL_FORMAT(video_st, video_st->codec), scale_mode, NULL, NULL, NULL);
 
 		// Add rescaler to vector
 		image_rescalers.push_back(img_convert_ctx);
