@@ -58,8 +58,9 @@ ReaderBase::ReaderBase()
 	info.channel_layout = LAYOUT_MONO;
 	info.audio_stream_index = -1;
 	info.audio_timebase = Fraction();
-	max_width = 0;
-	max_height = 0;
+
+	// Init parent clip
+	parent = NULL;
 }
 
 // Display file information
@@ -246,3 +247,13 @@ void ReaderBase::SetJsonValue(Json::Value root) {
 		}
 	}
 }
+
+/// Parent clip object of this reader (which can be unparented and NULL)
+ClipBase* ReaderBase::GetClip() {
+	return parent;
+}
+
+/// Set parent clip object of this reader
+void ReaderBase::SetClip(ClipBase* clip) {
+	parent = clip;
+}
